@@ -1,44 +1,37 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2012 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (c) 1998-2005 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *                 All Rights Reserved
+ *
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
- *   This program is free software; you can redistribute it and/or modify it
- *   under the terms of the GNU General Public License as published by the
- *   Free Software Foundation; either version 2 of the License, or (at your
- *   option) any later version.
- * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *   See the GNU General Public License for more details.
- * 
- *   You should have received a copy of the GNU General Public License along
- *   with this program; if not, see <http://www.gnu.org/licenses>.
- * 
- *   Linking stunnel statically or dynamically with other modules is making
- *   a combined work based on stunnel. Thus, the terms and conditions of
- *   the GNU General Public License cover the whole combination.
- * 
- *   In addition, as a special exception, the copyright holder of stunnel
- *   gives you permission to combine stunnel with free software programs or
- *   libraries that are released under the GNU LGPL and with code included
- *   in the standard release of OpenSSL under the OpenSSL License (or
- *   modified versions of such code, with unchanged license). You may copy
- *   and distribute such a system following the terms of the GNU GPL for
- *   stunnel and the licenses of the other code concerned.
- * 
- *   Note that people who make modified versions of stunnel are not obligated
- *   to grant this special exception for their modified versions; it is their
- *   choice whether to do so. The GNU General Public License gives permission
- *   to release a modified version without this exception; this exception
- *   also makes it possible to release a modified version which carries
- *   forward this exception.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *   In addition, as a special exception, Michal Trojnara gives
+ *   permission to link the code of this program with the OpenSSL
+ *   library (or with modified versions of OpenSSL that use the same
+ *   license as OpenSSL), and distribute linked combinations including
+ *   the two.  You must obey the GNU General Public License in all
+ *   respects for all of the code used other than OpenSSL.  If you modify
+ *   this file, you may extend this exception to your version of the
+ *   file, but you are not obligated to do so.  If you do not wish to
+ *   do so, delete this exception statement from your version.
  */
 
 #include "common.h"
 #include "prototypes.h"
 
-/**************************************** prototypes */
+/**************************************** Resolver functions */
 
 #ifndef HAVE_GETADDRINFO
 
@@ -48,15 +41,13 @@
 #ifndef EAI_NONAME
 #define EAI_NONAME 2
 #endif
-#ifndef EAI_SERVICE
-#define EAI_SERVICE 8
-#endif
 
-/* rename some potentially locally shadowed declarations */
+#ifdef USE_WIN32
+/* rename some locally shadowed declarations */
 #define getaddrinfo     local_getaddrinfo
 #define freeaddrinfo    local_freeaddrinfo
-
-#ifndef HAVE_STRUCT_ADDRINFO
+#define getnameinfo     local_getnameinfo
+#else /* defined(USE_WIN32) */
 struct addrinfo {
     int ai_flags;
     int ai_family;
@@ -67,7 +58,7 @@ struct addrinfo {
     char *ai_canonname;
     struct addrinfo *ai_next;
 };
-#endif
+#endif /* defined(USE_WIN32) */
 
 static int getaddrinfo(const char *, const char *,
     const struct addrinfo *, struct addrinfo **);
@@ -77,60 +68,24 @@ static void freeaddrinfo(struct addrinfo *);
 
 #endif /* !defined HAVE_GETADDRINFO */
 
-/**************************************** stunnel resolver API */
+static const char *s_gai_strerror(int);
 
-int name2addr(SOCKADDR_UNION *addr, char *name, char *default_host) {
-    SOCKADDR_LIST addr_list;
-    int retval;
-
-    addr_list.num=0;
-    addr_list.addr=NULL;
-    retval=name2addrlist(&addr_list, name, default_host);
-    if(retval>0)
-        memcpy(addr, &addr_list.addr[0], sizeof *addr);
-    if(addr_list.addr)
-        str_free(addr_list.addr);
-    return retval;
-}
-
-int hostport2addr(SOCKADDR_UNION *addr, char *hostname, char *portname) {
-    SOCKADDR_LIST addr_list;
-    int retval;
-
-    addr_list.num=0;
-    addr_list.addr=NULL;
-    retval=hostport2addrlist(&addr_list, hostname, portname);
-    if(retval>0)
-        memcpy(addr, &addr_list.addr[0], sizeof *addr);
-    if(addr_list.addr)
-        str_free(addr_list.addr);
-    return retval;
-}
-
-int name2addrlist(SOCKADDR_LIST *addr_list, char *name, char *default_host) {
-    char *tmp, *hostname, *portname;
-    int retval;
-
-    addr_list->cur=0; /* reset round-robin counter */
-
-    /* first check if this is a UNIX socket */
-#ifdef HAVE_STRUCT_SOCKADDR_UN
-    if(*name=='/') {
-        if(offsetof(struct sockaddr_un, sun_path)+strlen(name)+1
-                > sizeof(struct sockaddr_un)) {
-            s_log(LOG_ERR, "Unix socket path is too long");
-            return 0; /* no results */
-        }
-        addr_list->addr=str_realloc(addr_list->addr,
-            (addr_list->num+1)*sizeof(SOCKADDR_UNION));
-        addr_list->addr[addr_list->num].un.sun_family=AF_UNIX;
-        strcpy(addr_list->addr[addr_list->num].un.sun_path, name);
-        return ++(addr_list->num); /* ok - return the number of addresses */
-    }
+#ifndef HAVE_GETNAMEINFO
+#ifndef NI_NUMERICHOST
+#define NI_NUMERICHOST	2
+#endif
+#ifndef NI_NUMERICSERV
+#define NI_NUMERICSERV	8
+#endif
+static int getnameinfo(const struct sockaddr *, int,
+    char *, int , char *, int , int );
 #endif
 
+int name2addrlist(SOCKADDR_LIST *addr_list, char *name, char *default_host) {
+    char tmp[STRLEN], *hostname, *portname;
+
     /* set hostname and portname */
-    tmp=str_dup(name);
+    safecopy(tmp, name);
     portname=strrchr(tmp, ':');
     if(portname) {
         hostname=tmp;
@@ -141,9 +96,7 @@ int name2addrlist(SOCKADDR_LIST *addr_list, char *name, char *default_host) {
     }
 
     /* fill addr_list structure */
-    retval=hostport2addrlist(addr_list, hostname, portname);
-    str_free(tmp);
-    return retval;
+    return hostport2addrlist(addr_list, hostname, portname);
 }
 
 int hostport2addrlist(SOCKADDR_LIST *addr_list,
@@ -151,7 +104,9 @@ int hostport2addrlist(SOCKADDR_LIST *addr_list,
     struct addrinfo hints, *res=NULL, *cur;
     int err;
 
-    memset(&hints, 0, sizeof hints);
+    addr_list->cur=0; /* initialize round-robin counter */
+
+    memset(&hints, 0, sizeof(hints));
 #if defined(USE_IPv6) || defined(USE_WIN32)
     hints.ai_family=PF_UNSPEC;
 #else
@@ -159,113 +114,76 @@ int hostport2addrlist(SOCKADDR_LIST *addr_list,
 #endif
     hints.ai_socktype=SOCK_STREAM;
     hints.ai_protocol=IPPROTO_TCP;
-    do {
-        err=getaddrinfo(hostname, portname, &hints, &res);
-        if(err && res)
-            freeaddrinfo(res);
-        if(err==EAI_AGAIN) {
-            s_log(LOG_DEBUG, "getaddrinfo: EAI_AGAIN received: retrying");
-            sleep(1);
-        }
-    } while(err==EAI_AGAIN);
-    switch(err) {
-    case 0:
-        break; /* success */
-    case EAI_SERVICE:
-        s_log(LOG_ERR, "Unknown TCP service '%s'", portname);
-        return 0; /* error */
-    default:
+
+    err=getaddrinfo(hostname, portname, &hints, &res);
+    if(err) {
         s_log(LOG_ERR, "Error resolving '%s': %s",
             hostname, s_gai_strerror(err));
-        return 0; /* error */
+        if(res)
+            freeaddrinfo(res);
+        return 0; /* Error */
     }
-
     /* copy the list of addresses */
-    for(cur=res; cur; cur=cur->ai_next) {
-        if(cur->ai_addrlen>(int)sizeof(SOCKADDR_UNION)) {
+    cur=res;
+    while(cur && addr_list->num<MAX_HOSTS) {
+        if(cur->ai_addrlen>sizeof(SOCKADDR_UNION)) {
             s_log(LOG_ERR, "INTERNAL ERROR: ai_addrlen value too big");
             freeaddrinfo(res);
             return 0; /* no results */
         }
-        addr_list->addr=str_realloc(addr_list->addr,
-            (addr_list->num+1)*sizeof(SOCKADDR_UNION));
-        memcpy(&addr_list->addr[addr_list->num], cur->ai_addr, cur->ai_addrlen);
-        ++(addr_list->num);
+        memcpy(&addr_list->addr[addr_list->num],
+            cur->ai_addr, cur->ai_addrlen);
+        cur=cur->ai_next;
+        addr_list->num++;
     }
     freeaddrinfo(res);
     return addr_list->num; /* ok - return the number of addresses */
 }
 
-char *s_ntop(SOCKADDR_UNION *addr, socklen_t addrlen) {
-    int err;
-    char *host, *port, *retval;
+char *s_ntop(char *text, SOCKADDR_UNION *addr) {
+    char host[IPLEN-6], port[6];
 
-    if(addrlen==sizeof(u_short)) /* see UNIX(7) manual for details */
-        return str_dup("unnamed socket");
-    host=str_alloc(256);
-    port=str_alloc(256); /* needs to be long enough for AF_UNIX path */
-    err=getnameinfo(&addr->sa, addrlen,
-        host, 256, port, 256, NI_NUMERICHOST|NI_NUMERICSERV);
-    if(err) {
-        s_log(LOG_ERR, "getnameinfo: %s", s_gai_strerror(err));
-        retval=str_dup("unresolvable address");
-    } else
-        retval=str_printf("%s:%s", host, port);
-    str_free(host);
-    str_free(port);
-    return retval;
+    if(getnameinfo(&addr->sa, addr_len(*addr),
+            host, IPLEN-6, port, 6, NI_NUMERICHOST|NI_NUMERICSERV)) {
+        sockerror("getnameinfo");
+        strcpy(text, "unresolvable IP");
+        return text;
+    }
+    strcpy(text, host);
+    strcat(text, ":");
+    strcat(text, port);
+    return text;
 }
 
-socklen_t addr_len(const SOCKADDR_UNION *addr) {
-    if(addr->sa.sa_family==AF_INET)
-        return sizeof(struct sockaddr_in);
-#ifdef USE_IPv6
-    if(addr->sa.sa_family==AF_INET6)
-        return sizeof(struct sockaddr_in6);
-#endif
-#ifdef HAVE_STRUCT_SOCKADDR_UN
-    if(addr->sa.sa_family==AF_UNIX)
-        return sizeof(struct sockaddr_un);
-#endif
-    s_log(LOG_ERR, "INTERNAL ERROR: Unknown sa_family: %d",
-        addr->sa.sa_family);
-    return sizeof(SOCKADDR_UNION);
-}
-
-/**************************************** my getaddrinfo() */
-/* implementation is limited to functionality needed by stunnel */
+/**************************************** My getaddrinfo() and getnameinfo() */
+/* implementations are limited to functionality needed by stunnel */
 
 #ifndef HAVE_GETADDRINFO
 static int getaddrinfo(const char *node, const char *service,
         const struct addrinfo *hints, struct addrinfo **res) {
     struct hostent *h;
-#ifndef _WIN32_WCE
     struct servent *p;
-#endif
     u_short port;
     struct addrinfo *ai;
     int retval;
-    char *tmpstr;
 
-#if defined(USE_WIN32) && !defined(_WIN32_WCE)
+#ifdef USE_WIN32
     if(s_getaddrinfo)
         return s_getaddrinfo(node, service, hints, res);
 #endif
     /* decode service name */
-    port=htons((u_short)strtol(service, &tmpstr, 10));
-    if(tmpstr==service || *tmpstr) { /* not a number */
-#ifdef _WIN32_WCE
-        return EAI_NONAME;
-#else /* defined(_WIN32_WCE) */
+    port=htons((u_short)atoi(service));
+    if(!port) { /* zero is an illegal value for port number */
         p=getservbyname(service, "tcp");
         if(!p)
             return EAI_NONAME;
         port=p->s_port;
-#endif /* defined(_WIN32_WCE) */
     }
 
     /* allocate addrlist structure */
-    ai=str_alloc(sizeof(struct addrinfo));
+    ai=calloc(1, sizeof(struct addrinfo));
+    if(!ai)
+        return EAI_MEMORY;
     if(hints)
         memcpy(ai, hints, sizeof(struct addrinfo));
 
@@ -273,14 +191,22 @@ static int getaddrinfo(const char *node, const char *service,
 #if defined(USE_IPv6) && !defined(USE_WIN32)
     ai->ai_family=AF_INET6;
     ai->ai_addrlen=sizeof(struct sockaddr_in6);
-    ai->ai_addr=str_alloc(ai->ai_addrlen);
+    ai->ai_addr=calloc(1, ai->ai_addrlen);
+    if(!ai->ai_addr) {
+        free(ai);
+        return EAI_MEMORY;
+    }
     ai->ai_addr->sa_family=AF_INET6;
     if(inet_pton(AF_INET6, node,
             &((struct sockaddr_in6 *)ai->ai_addr)->sin6_addr)>0) {
 #else
     ai->ai_family=AF_INET;
     ai->ai_addrlen=sizeof(struct sockaddr_in);
-    ai->ai_addr=str_alloc(ai->ai_addrlen);
+    ai->ai_addr=calloc(1, ai->ai_addrlen);
+    if(!ai->ai_addr) {
+        free(ai);
+        return EAI_MEMORY;
+    }
     ai->ai_addr->sa_family=AF_INET;
     ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr=inet_addr(node);
     if(((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr+1) {
@@ -290,14 +216,14 @@ static int getaddrinfo(const char *node, const char *service,
         *res=ai;
         return 0; /* numerical address resolved */
     }
-    str_free(ai->ai_addr);
-    str_free(ai);
+    free(ai->ai_addr);
+    free(ai);
 
     /* not numerical: need to call resolver library */
     *res=NULL;
     ai=NULL;
     enter_critical_section(CRIT_INET);
-#ifdef HAVE_GETHOSTBYNAME2
+#if defined(USE_IPv6) && !defined(USE_WIN32)
     h=gethostbyname2(node, AF_INET6);
     if(h) /* some IPv6 addresses found */
         alloc_addresses(h, hints, port, res, &ai); /* ignore the error */
@@ -329,7 +255,9 @@ static int alloc_addresses(struct hostent *h, const struct addrinfo *hints,
 
     /* copy addresses */
     for(i=0; h->h_addr_list[i]; i++) {
-        ai=str_alloc(sizeof(struct addrinfo));
+        ai=calloc(1, sizeof(struct addrinfo));
+        if(!ai)
+            return EAI_MEMORY;
         if(hints)
             memcpy(ai, hints, sizeof(struct addrinfo));
         ai->ai_next=NULL; /* just in case */
@@ -344,14 +272,18 @@ static int alloc_addresses(struct hostent *h, const struct addrinfo *hints,
 #if defined(USE_IPv6)
         if(h->h_addrtype==AF_INET6) {
             ai->ai_addrlen=sizeof(struct sockaddr_in6);
-            ai->ai_addr=str_alloc(ai->ai_addrlen);
+            ai->ai_addr=calloc(1, ai->ai_addrlen);
+            if(!ai->ai_addr)
+                return EAI_MEMORY;
             memcpy(&((struct sockaddr_in6 *)ai->ai_addr)->sin6_addr,
                 h->h_addr_list[i], h->h_length);
         } else
 #endif
         {
             ai->ai_addrlen=sizeof(struct sockaddr_in);
-            ai->ai_addr=str_alloc(ai->ai_addrlen);
+            ai->ai_addr=calloc(1, ai->ai_addrlen);
+            if(!ai->ai_addr)
+                return EAI_MEMORY;
             memcpy(&((struct sockaddr_in *)ai->ai_addr)->sin_addr,
                 h->h_addr_list[i], h->h_length);
         }
@@ -365,26 +297,25 @@ static int alloc_addresses(struct hostent *h, const struct addrinfo *hints,
 static void freeaddrinfo(struct addrinfo *current) {
     struct addrinfo *next;
 
-#if defined(USE_WIN32) && !defined(_WIN32_WCE)
-    if(s_freeaddrinfo) {
-        s_freeaddrinfo(current);
-    return;
-    }
+#ifdef USE_WIN32
+    if(s_freeaddrinfo)
+        return s_freeaddrinfo(current);
 #endif
     while(current) {
         if(current->ai_addr)
-            str_free(current->ai_addr);
+            free(current->ai_addr);
         if(current->ai_canonname)
-            str_free(current->ai_canonname);
+            free(current->ai_canonname);
         next=current->ai_next;
-        str_free(current);
+        free(current);
         current=next;
     }
 }
+
 #endif /* !defined HAVE_GETADDRINFO */
 
-/* due to a problem with Mingw32 I decided to define my own gai_strerror() */
-const char *s_gai_strerror(int err) {
+/* Due to a problem with Mingw32 I decided to define my own gai_strerror() */
+static const char *s_gai_strerror(int err) {
     switch(err) {
 #ifdef EAI_BADFLAGS
         case EAI_BADFLAGS:
@@ -433,20 +364,17 @@ const char *s_gai_strerror(int err) {
     }
 }
 
-/**************************************** my getnameinfo() */
-/* implementation is limited to functionality needed by stunnel */
-
 #ifndef HAVE_GETNAMEINFO
-int getnameinfo(const struct sockaddr *sa, int salen,
+static int getnameinfo(const struct sockaddr *sa, int salen,
     char *host, int hostlen, char *serv, int servlen, int flags) {
 
-#if defined(USE_WIN32) && !defined(_WIN32_WCE)
+#ifdef USE_WIN32
     if(s_getnameinfo)
         return s_getnameinfo(sa, salen, host, hostlen, serv, servlen, flags);
 #endif
     if(host && hostlen) {
-#if defined(USE_IPv6) && !defined(USE_WIN32)
-        inet_ntop(sa->sa_family, sa->sa_family==AF_INET6 ?
+#ifdef USE_IPv6
+        inet_ntop(sa->sa_family, sa->sa_family==AF_INET6 ? 
                 (void *)&((struct sockaddr_in6 *)sa)->sin6_addr :
                 (void *)&((struct sockaddr_in *)sa)->sin_addr,
             host, hostlen);
@@ -466,4 +394,22 @@ int getnameinfo(const struct sockaddr *sa, int salen,
 }
 #endif
 
-/* end of resolver.c */
+/**************************************** Some optional debugging stuff */
+
+#if 0
+void debug_sockaddr_union(SOCKADDR_UNION *addr) {
+    int i;
+    char t[3*sizeof(SOCKADDR_UNION)];
+    const char hex[16]="0123456789abcdef";
+    
+    for(i=0; i<sizeof(SOCKADDR_UNION); i++) {
+        t[3*i]=hex[((u8 *)addr)[i]/16];
+        t[3*i+1]=hex[((u8 *)addr)[i]%16];
+        t[3*i+2]=' ';
+    }
+    t[3*sizeof(SOCKADDR_UNION)-1]='\0';
+    s_log(LOG_DEBUG, "%s\n", t);
+}
+#endif
+
+/* End of resolver.c */
