@@ -3,8 +3,8 @@
  *   Copyright (c) 1998-2005 Michal Trojnara <Michal.Trojnara@mirt.net>
  *                 All Rights Reserved
  *
- *   Version:      4.09             (stunnel.c)
- *   Date:         2005.03.26
+ *   Version:      4.14             (stunnel.c)
+ *   Date:         2005.11.02
  *
  *   Author:       Michal Trojnara  <Michal.Trojnara@mirt.net>
  *
@@ -81,6 +81,7 @@ int main(int argc, char* argv[]) { /* execution begins here 8-) */
 void main_initialize(char *arg1, char *arg2) {
     struct stat st; /* buffer for stat */
 
+    ssl_init(); /* initialize SSL library */
     sthreads_init(); /* initialize critical sections & SSL callbacks */
     parse_config(arg1, arg2);
     log_open();
@@ -102,7 +103,7 @@ void main_initialize(char *arg1, char *arg2) {
 }
 
 void main_execute(void) {
-    ssl_init(); /* initialize SSL library */
+    ssl_configure(); /* configure global SSL settings */
     context_init(); /* initialize global SSL context */
     /* check if started from inetd */
     if(local_options.next) { /* there are service sections -> daemon mode */
@@ -201,8 +202,10 @@ static void daemon_loop(void) {
 static void accept_connection(LOCAL_OPTIONS *opt) {
     SOCKADDR_UNION addr;
     char from_address[IPLEN];
-    int s, addrlen=sizeof(SOCKADDR_UNION);
+    int s;
+    socklen_t addrlen;
 
+    addrlen=sizeof(SOCKADDR_UNION);
     while((s=accept(opt->fd, &addr.sa, &addrlen))<0) {
         switch(get_last_socket_error()) {
             case EINTR:
@@ -228,8 +231,6 @@ static void accept_connection(LOCAL_OPTIONS *opt) {
         closesocket(s);
         return;
     }
-    if(alloc_fd(s))
-        return;
 #ifdef FD_CLOEXEC
     fcntl(s, F_SETFD, FD_CLOEXEC); /* close socket in child execvp */
 #endif
@@ -265,7 +266,7 @@ static void get_limits(void) {
 #endif
     s_log(LOG_INFO, "file ulimit = %d%s (can be changed with 'ulimit -n')",
         max_fds, max_fds ? "" : " (unlimited)");
-#ifdef HAVE_POLL
+#ifdef USE_POLL
     s_log(LOG_INFO, "poll() used - no FD_SETSIZE limit for file descriptors");
 #else
     s_log(LOG_INFO,
@@ -434,6 +435,9 @@ char *stunnel_info(void) {
     static char retval[STRLEN];
 
     safecopy(retval, "stunnel " VERSION " on " HOST);
+#ifdef USE_UCONTEXT
+    safeconcat(retval, " UCONTEXT");
+#endif
 #ifdef USE_PTHREAD
     safeconcat(retval, " PTHREAD");
 #endif
@@ -443,9 +447,11 @@ char *stunnel_info(void) {
 #ifdef USE_FORK
     safeconcat(retval, " FORK");
 #endif
-#ifdef HAVE_POLL
+#ifdef USE_POLL
     safeconcat(retval, "+POLL");
-#endif
+#else /* defined(USE_POLL) */
+    safeconcat(retval, "+SELECT");
+#endif /* defined(USE_POLL) */
 #ifdef USE_WIN32
     if(s_getaddrinfo)
         safeconcat(retval, "+IPv6");
